<?php

if (!defined('_PS_VERSION_')) {
    exit;
}

define('make_influence_pixel_src', 'https://system.makeinfluence.com/p');
define('make_influence_server_side_tracking', 'https://system.makeinfluence.com/track-conversion');

class MakeInfluencePixel extends Module
{
    public function __construct()
    {
        $this->name          = 'makeinfluencepixel';
        $this->tab           = 'advertising_marketing';
        $this->version       = '1.4';
        $this->author        = 'Modified Solutions';
        $this->author_uri    = 'modified.dk';
        $this->need_instance = 0;
        $this->bootstrap     = true;

        parent::__construct();

        $this->displayName = $this->l('Make Influence Tracking Pixel');
        $this->description = $this->l('Allow tracking with Make Influence');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
        $this->ps_versions_compliancy = array('min' => '1.5', 'max' => '1.8');
    }

    public function install()
    {
        return parent::install() &&    
            $this->registerHook('displayOrderConfirmation');    // Run after order was confirmed.
    }

    public function uninstall()
    {
        if (!parent::uninstall()) {
            return false;
        }
        return true;
    }

    public function displayConfigForm()
    {
        $helper = new HelperForm();
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;
        $helper->toolbar_scroll = true;
        $helper->submit_action = 'submit' . $this->name;
        $helper->toolbar_btn = array(
            'save' =>
            array(
                'title' => $this->l('Save'),
                'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
                          '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'class' => 'btn btn-default pull-right'
            ),
            'back' => array(
                'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );

        // Define the elements displayed in the configuration menu.

        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Make Influence Tracking Pixel'),
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('Business ID:'),
                    'name' => 'make_influence_business_id',
                    'size' => 20,
                    'required' => true,
                    'desc' => $this->l('Type the Business ID provided by Make Influence here in order to start tracking sales.'),
                )
            ),
            'submit' => array(
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right'
            ),

        );

        $helper->fields_value['make_influence_business_id'] = Configuration::get('make_influence_business_id');

        return $helper->generateForm($fields_form);
    }

    public function getContent()
    {
        $output = null;
 
        if (Tools::isSubmit('submit' . $this->name))
        {
            Configuration::updateValue('make_influence_business_id', Tools::getValue('make_influence_business_id'));
            $output .= $this->displayConfirmation($this->l('Settings updated'));
        }
        return $output . $this->displayConfigForm();
    }

    private function getPixelParams($businessId, $orderId)
    {
        $order = new Order($orderId);
        $orderCurrency = (new CurrencyCore($order->id_currency))->iso_code;

        /**
         * total_paid_tax_incl is the total order price with shipping cost.
         * We don't want to include the shipping cost.
         **/
        $orderTotal = number_format($order->total_paid_tax_incl - $order->total_shipping_tax_incl, 2, '.', '');
        $cart = new Cart($order->id_cart);
        $cartRules = $cart->getCartRules();
        $discountRules = [];

        foreach ($cartRules as $rule) {
            $discountRules[] = $rule['code'];
        }

        $promotionCodes = implode(",", $discountRules);

        $params = [
            'bid' => $businessId,
            'uid' => $orderId,
            'value' => $orderTotal,
            'promotion_code' => $promotionCodes,
            'currency' => $orderCurrency
        ];

        return [$params, $order];
    }

    private function serverSideTracking(array $pixelParams, $order)
    {
        if (!$order) {
            return;
        }

        $postParams = [
            'business_id' => $pixelParams['bid'],
            'unique_id' => $pixelParams['uid'],
            'value' => $pixelParams['value'],
            'promotion_code' => $pixelParams['promotion_code'],
            'created_at' => gmdate('Y-m-d H:i:s', strtotime($order->date_add)), // Converted to utc+0.
            'currency' => $pixelParams['currency'],
            'ip' => isset($_SERVER['REMOTE_ADDR']) ? strval($_SERVER['REMOTE_ADDR']) : '',
            'user_agent' => isset($_SERVER['HTTP_USER_AGENT']) ? strval($_SERVER['HTTP_USER_AGENT']) : '',
            'http_referer' => isset($_SERVER["HTTP_REFERER"]) ? strval($_SERVER["HTTP_REFERER"]) : _PS_BASE_URL_
        ];

        // Use curl if it exists.
        if (function_exists('curl_version')) {
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, make_influence_server_side_tracking);
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postParams));
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            $server_output = curl_exec($ch);
            curl_close ($ch);
        } else {
            // default to use file_get_contents.
            
            $context  = stream_context_create([
                'http' => [
                        'method'  => 'POST',
                        'header'  => 'Content-Type: application/x-www-form-urlencoded',
                        'content' => http_build_query($postParams)
                    ]
                ]
            );
            $result = @file_get_contents(make_influence_server_side_tracking, false, $context);

            if (!$result) {
                $this->writeLog("Make Influence Pixel - submission failed!", 3);
            }
        }
    }

    /**
     * $severity 1 is info
     * $severity 2 is warning
     * $severity 3 is error
     * $severity 4 is major issue
     */
    private function writeLog($text, $severity = 1)
    {
        if (method_exists('PrestaShopLogger','addLog')) {   // Later than Prestashop 1.5
            PrestaShopLogger::addLog($text, $severity);
        } else if (method_exists('Logger','addLog')) {      // Prestashop 1.5
            Logger::addLog($text, $severity);
        }
    }

    /**
     * Generate the pixel parameters and show pixel in header on order
     * confirmation page.
     */
    public function hookDisplayOrderConfirmation(array $params)
    {
        $businessId = Configuration::get('make_influence_business_id');
        
        // Don't do anything if there's no business id.
        if (!$businessId) {
            $this->writeLog("Make Influence Pixel - Error: businessId not found!", 3);
            return;
        }

        // Prestashop 1.7
        $orderId = (isset($params['order']) && isset($params['order']->id)) ? $params['order']->id : null;

        // Prestashop 1.6
        if (!$orderId) {
            $orderId = (isset($params['objOrder']) && isset($params['objOrder']->id)) ? $params['objOrder']->id : null;
        }

        // Don't do anything if there's no order id.
        if (!$orderId) {
            $this->writeLog("Make Influence Pixel - Error: orderId not found!", 3);
            return;
        }

        list($pixelParams, $order) = $this->getPixelParams($businessId, $orderId);

        $this->serverSideTracking($pixelParams, $order);

        $this->smarty->assign(array(
            'pixelParams' => http_build_query($pixelParams),
            'pixelSrc' => make_influence_pixel_src
        ));

        /**
         * Reference:
         * <img src="https://makeinfluence.com/p?bid=BUSINESS_ID&uid=ORDER_ID&value=ORDER_TOTAL&promotion_code=PROMOTION_CODE&currency=ORDER_CURRENY">
         */
        return $this->display(__FILE__, 'template.tpl');
    }
}
